@@ -1,5 +1,5 @@
 /* GIO - GLib Input, Output and Streaming Library
- * 
+ *
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -48,7 +48,7 @@ typedef struct _GAppLaunchContextPrivate GAppLaunchContextPrivate;
 
 /**
  * GAppInfo:
- * 
+ *
  * Information about an installed application and methods to launch
  * it (with file arguments).
  */
@@ -75,7 +75,9 @@ typedef struct _GAppLaunchContextPrivate GAppLaunchContextPrivate;
  * @add_supports_type: Adds to the #GAppInfo information about supported file types.
  * @can_remove_supports_type: Checks for support for removing supported file types from a #GAppInfo.
  * @remove_supports_type: Removes a supported application type from a #GAppInfo.
- * 
+ * @can_delete: Checks if a #GAppInfo can be deleted. Since 2.20
+ * @do_delete: Deletes a #GAppInfo. Since 2.20
+ *
  * Application Information interface, for operating system portability.
  */
 typedef struct _GAppInfoIface    GAppInfoIface;
@@ -86,45 +88,45 @@ struct _GAppInfoIface
 
   /* Virtual Table */
 
-  GAppInfo *          (*dup)                (GAppInfo                *appinfo);
-  gboolean            (*equal)              (GAppInfo                *appinfo1,
-					     GAppInfo                *appinfo2);
-  const char *        (*get_id)             (GAppInfo                *appinfo);
-  const char *        (*get_name)           (GAppInfo                *appinfo);
-  const char *        (*get_description)    (GAppInfo                *appinfo);
-  const char *        (*get_executable)     (GAppInfo                *appinfo);
-  GIcon *             (*get_icon)           (GAppInfo                *appinfo);
-  gboolean            (*launch)             (GAppInfo                *appinfo,
-					     GList                   *filenames,
-					     GAppLaunchContext       *launch_context,
-					     GError                 **error);
-  gboolean            (*supports_uris)      (GAppInfo                *appinfo);
-  gboolean            (*supports_files)     (GAppInfo                *appinfo);
-  gboolean            (*launch_uris)        (GAppInfo                *appinfo,
-					     GList                   *uris,
-					     GAppLaunchContext       *launch_context,
-					     GError                 **error);
-  gboolean            (*should_show)        (GAppInfo                *appinfo);
+  GAppInfo *   (* dup)                          (GAppInfo           *appinfo);
+  gboolean     (* equal)                        (GAppInfo           *appinfo1,
+                                                 GAppInfo           *appinfo2);
+  const char * (* get_id)                       (GAppInfo           *appinfo);
+  const char * (* get_name)                     (GAppInfo           *appinfo);
+  const char * (* get_description)              (GAppInfo           *appinfo);
+  const char * (* get_executable)               (GAppInfo           *appinfo);
+  GIcon *      (* get_icon)                     (GAppInfo           *appinfo);
+  gboolean     (* launch)                       (GAppInfo           *appinfo,
+                                                 GList              *filenames,
+                                                 GAppLaunchContext  *launch_context,
+                                                 GError            **error);
+  gboolean     (* supports_uris)                (GAppInfo           *appinfo);
+  gboolean     (* supports_files)               (GAppInfo           *appinfo);
+  gboolean     (* launch_uris)                  (GAppInfo           *appinfo,
+                                                 GList              *uris,
+                                                 GAppLaunchContext  *launch_context,
+                                                 GError            **error);
+  gboolean     (* should_show)                  (GAppInfo           *appinfo);
 
   /* For changing associations */
-  gboolean  (*set_as_default_for_type)      (GAppInfo           *appinfo,
-					     const char         *content_type,
-					     GError            **error);
-  gboolean  (*set_as_default_for_extension) (GAppInfo           *appinfo,
-					     const char         *extension,
-					     GError            **error);
-  gboolean  (*add_supports_type)            (GAppInfo           *appinfo,
-					     const char         *content_type,
-					     GError            **error);
-  gboolean  (*can_remove_supports_type)     (GAppInfo           *appinfo);
-  gboolean  (*remove_supports_type)         (GAppInfo           *appinfo,
-					     const char         *content_type,
-					     GError            **error);
+  gboolean     (* set_as_default_for_type)      (GAppInfo           *appinfo,
+                                                 const char         *content_type,
+                                                 GError            **error);
+  gboolean     (* set_as_default_for_extension) (GAppInfo           *appinfo,
+                                                 const char         *extension,
+                                                 GError            **error);
+  gboolean     (* add_supports_type)            (GAppInfo           *appinfo,
+                                                 const char         *content_type,
+                                                 GError            **error);
+  gboolean     (* can_remove_supports_type)     (GAppInfo           *appinfo);
+  gboolean     (* remove_supports_type)         (GAppInfo           *appinfo,
+                                                 const char         *content_type,
+                                                 GError            **error);
+  gboolean     (* can_delete)                   (GAppInfo           *appinfo);
+  gboolean     (* do_delete)                    (GAppInfo           *appinfo);
 };
 
-GType g_app_info_get_type (void) G_GNUC_CONST;
-GType g_app_launch_context_get_type (void) G_GNUC_CONST;
-
+GType       g_app_info_get_type                     (void) G_GNUC_CONST;
 GAppInfo *  g_app_info_create_from_commandline      (const char           *commandline,
 						     const char           *application_name,
 						     GAppInfoCreateFlags   flags,
@@ -162,9 +164,12 @@ gboolean    g_app_info_can_remove_supports_type     (GAppInfo             *appin
 gboolean    g_app_info_remove_supports_type         (GAppInfo             *appinfo,
 						     const char           *content_type,
 						     GError              **error);
+gboolean    g_app_info_can_delete                   (GAppInfo   *appinfo);
+gboolean    g_app_info_delete                       (GAppInfo   *appinfo);
 
 GList *   g_app_info_get_all                     (void);
 GList *   g_app_info_get_all_for_type            (const char  *content_type);
+void      g_app_info_reset_type_associations     (const char  *content_type);
 GAppInfo *g_app_info_get_default_for_type        (const char  *content_type,
 						  gboolean     must_support_uris);
 GAppInfo *g_app_info_get_default_for_uri_scheme  (const char  *uri_scheme);
@@ -176,7 +181,7 @@ gboolean  g_app_info_launch_default_for_uri      (const char              *uri,
 /**
  * GAppLaunchContext:
  * @parent_instance: The parent instance.
- * 
+ *
  * Integrating the launch with the launching application. This is used to
  * handle for instance startup notification and launching the new application
  * on the same screen as the launching window.
@@ -193,14 +198,14 @@ struct _GAppLaunchContextClass
 {
   GObjectClass parent_class;
 
-  char * (*get_display)           (GAppLaunchContext *context,
-				   GAppInfo *info,
-				   GList *files);
-  char * (*get_startup_notify_id) (GAppLaunchContext *context,
-				   GAppInfo *info,
-				   GList *files);
-  void   (*launch_failed)         (GAppLaunchContext *context,
-				   const char *startup_notify_id);
+  char * (* get_display)           (GAppLaunchContext *context,
+                                    GAppInfo          *info,
+                                    GList             *files);
+  char * (* get_startup_notify_id) (GAppLaunchContext *context,
+                                    GAppInfo          *info,
+                                    GList             *files);
+  void   (* launch_failed)         (GAppLaunchContext *context,
+                                    const char        *startup_notify_id);
 
   /* Padding for future expansion */
   void (*_g_reserved1) (void);
@@ -210,6 +215,7 @@ struct _GAppLaunchContextClass
   void (*_g_reserved5) (void);
 };
 
+GType              g_app_launch_context_get_type              (void) G_GNUC_CONST;
 GAppLaunchContext *g_app_launch_context_new                   (void);
 char *             g_app_launch_context_get_display           (GAppLaunchContext *context,
 							       GAppInfo          *info,
