@@ -61,7 +61,8 @@
 #define MIME_CACHE_GROUP            "MIME Cache"
 
 static void     g_desktop_app_info_iface_init         (GAppInfoIface    *iface);
-static GList *  get_all_desktop_entries_for_mime_type (const char       *base_mime_type);
+static GList *  get_all_desktop_entries_for_mime_type (const char       *base_mime_type,
+						       const char      **except);
 static void     mime_info_cache_reload                (const char       *dir);
 static gboolean g_desktop_app_info_ensure_saved       (GDesktopAppInfo  *info,
 						       GError          **error);
@@ -272,12 +273,18 @@ g_desktop_app_info_new_from_keyfile (GKeyFile *key_file)
   if (info->exec)
     info->binary = binary_from_exec (info->exec);
   
+  if (info->path && info->path[0] == '\0')
+    {
+      g_free (info->path);
+      info->path = NULL;
+    }
+
   return info;
 }
 
 /**
  * g_desktop_app_info_new_from_filename:
- * @filename: a string containing a file name.
+ * @filename: the path of a desktop file, in the GLib filename encoding
  * 
  * Creates a new #GDesktopAppInfo.
  *
@@ -474,7 +481,7 @@ expand_macro_single (char macro, char *uri)
 {
   GFile *file;
   char *result = NULL;
-  char *path;
+  char *path, *name;
 
   file = g_file_new_for_uri (uri);
   path = g_file_get_path (file);
@@ -494,12 +501,20 @@ expand_macro_single (char macro, char *uri)
     case 'd':
     case 'D':
       if (path)
-	result = g_shell_quote (g_path_get_dirname (path));
+        {
+          name = g_path_get_dirname (path);
+	  result = g_shell_quote (name);
+          g_free (name);
+        }
       break;
     case 'n':
     case 'N':
       if (path)
-	result = g_shell_quote (g_path_get_basename (path));
+        {
+          name = g_path_get_basename (path);
+	  result = g_shell_quote (name);
+          g_free (name);
+        }
       break;
     }
 
@@ -516,9 +531,32 @@ expand_macro (char              macro,
 {
   GList *uris = *uri_list;
   char *expanded;
-  
+  gboolean force_file_uri;
+  char force_file_uri_macro;
+
   g_return_if_fail (exec != NULL);
-  
+
+  /* On %u and %U, pass POSIX file path pointing to the URI via
+   * the FUSE mount in ~/.gvfs. Note that if the FUSE daemon isn't
+   * running or the URI doesn't have a POSIX file path via FUSE
+   * we'll just pass the URI.
+   */
+  switch (macro)
+    {
+    case 'u':
+      force_file_uri_macro = 'f';
+      force_file_uri = TRUE;
+      break;
+    case 'U':
+      force_file_uri_macro = 'F';
+      force_file_uri = TRUE;
+      break;
+    default:
+      force_file_uri_macro = macro;
+      force_file_uri = FALSE;
+      break;
+    }
+
   switch (macro)
     {
     case 'u':
@@ -527,7 +565,17 @@ expand_macro (char              macro,
     case 'n':
       if (uris)
 	{
-	  expanded = expand_macro_single (macro, uris->data);
+          if (!force_file_uri)
+            {
+              expanded = expand_macro_single (macro, uris->data);
+            }
+          else
+            {
+              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              if (expanded == NULL)
+                expanded = expand_macro_single (macro, uris->data);
+            }
+
 	  if (expanded)
 	    {
 	      g_string_append (exec, expanded);
@@ -544,7 +592,17 @@ expand_macro (char              macro,
     case 'N':
       while (uris)
 	{
-	  expanded = expand_macro_single (macro, uris->data);
+          if (!force_file_uri)
+            {
+              expanded = expand_macro_single (macro, uris->data);
+            }
+          else
+            {
+              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              if (expanded == NULL)
+                expanded = expand_macro_single (macro, uris->data);
+            }
+
 	  if (expanded)
 	    {
 	      g_string_append (exec, expanded);
@@ -1145,22 +1203,23 @@ ensure_dir (DirType   type,
 static gboolean
 update_mimeapps_list (const char  *desktop_id, 
 		      const char  *content_type, 
-		      gboolean     add_at_start, 
-		      gboolean     add_at_end, 
+		      gboolean     add_as_default,
+		      gboolean     add_non_default,
 		      gboolean     remove, 
 		      GError     **error)
 {
   char *dirname, *filename;
   GKeyFile *key_file;
   gboolean load_succeeded, res;
-  char **old_list;
-  char **list;
+  char **old_list, **list;
+  GList *system_list, *l;
   gsize length, data_size;
   char *data;
-  int i, j;
+  int i, j, k;
+  char **content_types;
 
   /* Don't add both at start and end */
-  g_assert (!(add_at_start && add_at_end));
+  g_assert (!(add_as_default && add_non_default));
   
   dirname = ensure_dir (APP_DIR, error);
   if (!dirname)
@@ -1177,75 +1236,139 @@ update_mimeapps_list (const char  *desktop_id,
       key_file = g_key_file_new ();
     }
 
-  /* Add to the right place in the list */
-  
-  length = 0;
-  old_list = g_key_file_get_string_list (key_file, ADDED_ASSOCIATIONS_GROUP,
-					 content_type, &length, NULL);
-
-  list = g_new (char *, 1 + length + 1);
-
-  i = 0;
-  if (add_at_start)
-    list[i++] = g_strdup (desktop_id);
-  if (old_list)
+  if (content_type)
     {
-      for (j = 0; old_list[j] != NULL; j++)
-	{
-	  if (strcmp (old_list[j], desktop_id) != 0)
-	    list[i++] = g_strdup (old_list[j]);
-	}
+      content_types = g_new (char *, 2);
+      content_types[0] = g_strdup (content_type);
+      content_types[1] = NULL;
     }
-  if (add_at_end)
-    list[i++] = g_strdup (desktop_id);
-  list[i] = NULL;
-  
-  g_strfreev (old_list);
-
-  g_key_file_set_string_list (key_file,
-			      ADDED_ASSOCIATIONS_GROUP,
-			      content_type,
-			      (const char * const *)list, i);
-
-  g_strfreev (list);
-
-  /* Remove from removed associations group (unless remove) */
-  
-  length = 0;
-  old_list = g_key_file_get_string_list (key_file, REMOVED_ASSOCIATIONS_GROUP,
-					 content_type, &length, NULL);
-
-  list = g_new (char *, 1 + length + 1);
-
-  i = 0;
-  if (remove)
-    list[i++] = g_strdup (desktop_id);
-  if (old_list)
-    {
-      for (j = 0; old_list[j] != NULL; j++)
-	{
-	  if (strcmp (old_list[j], desktop_id) != 0)
-	    list[i++] = g_strdup (old_list[j]);
-	}
-    }
-  list[i] = NULL;
-  
-  g_strfreev (old_list);
-
-  if (list[0] == NULL)
-    g_key_file_remove_key (key_file,
-			   REMOVED_ASSOCIATIONS_GROUP,
-			   content_type,
-			   NULL);
   else
-    g_key_file_set_string_list (key_file,
-				REMOVED_ASSOCIATIONS_GROUP,
-				content_type,
-				(const char * const *)list, i);
-
-  g_strfreev (list);
+    {
+      content_types = g_key_file_get_keys (key_file, ADDED_ASSOCIATIONS_GROUP, NULL, NULL);
+    }
 
+  for (k = 0; content_types && content_types[k]; k++)
+    { 
+      /* Add to the right place in the list */
   
+      length = 0;
+      old_list = g_key_file_get_string_list (key_file, ADDED_ASSOCIATIONS_GROUP,
+					     content_types[k], &length, NULL);
+
+      list = g_new (char *, 1 + length + 1);
+
+      i = 0;
+      if (add_as_default)
+        list[i++] = g_strdup (desktop_id);
+      if (old_list)
+        {
+          for (j = 0; old_list[j] != NULL; j++)
+	    {
+	      if (g_strcmp0 (old_list[j], desktop_id) != 0)
+	        list[i++] = g_strdup (old_list[j]);
+	      else if (add_non_default)
+		{
+		  /* If adding as non-default, and its already in,
+		     don't change order of desktop ids */
+		  add_non_default = FALSE;
+		  list[i++] = g_strdup (old_list[j]);
+		}
+	    }
+        }
+      
+      if (add_non_default)
+	{
+	  /* We're adding as non-default, and it wasn't already in the list,
+	     so we add at the end. But to avoid listing the app before the
+	     current system default (thus changing the default) we have to
+	     add the current list of (not yet listed) apps before it. */
+
+	  list[i] = NULL; /* Terminate current list so we can use it */
+	  system_list =  get_all_desktop_entries_for_mime_type (content_type, list);
+	  
+	  list = g_renew (char *, list, 1 + length + g_list_length (system_list) + 1);
+	  
+	  for (l = system_list; l != NULL; l = l->next)
+	    {
+	      list[i++] = l->data; /* no strdup, taking ownership */
+	      if (g_strcmp0 (l->data, desktop_id) == 0)
+		add_non_default = FALSE;
+	    }
+	  g_list_free (system_list);
+		  
+	  if (add_non_default)
+	    list[i++] = g_strdup (desktop_id);
+	}
+      
+      list[i] = NULL;
+  
+      g_strfreev (old_list);
+
+      if (list[0] == NULL || desktop_id == NULL)
+        g_key_file_remove_key (key_file,
+			       ADDED_ASSOCIATIONS_GROUP,
+			       content_types[k],
+			       NULL);
+      else
+        g_key_file_set_string_list (key_file,
+			            ADDED_ASSOCIATIONS_GROUP,
+			            content_types[k],
+			            (const char * const *)list, i);
+   
+      g_strfreev (list);
+    }
+  
+  if (content_type)
+    {
+      /* reuse the list from above */
+    }
+  else
+    {
+      g_strfreev (content_types);
+      content_types = g_key_file_get_keys (key_file, REMOVED_ASSOCIATIONS_GROUP, NULL, NULL);
+    }
+
+  for (k = 0; content_types && content_types[k]; k++) 
+    {
+      /* Remove from removed associations group (unless remove) */
+  
+      length = 0;
+      old_list = g_key_file_get_string_list (key_file, REMOVED_ASSOCIATIONS_GROUP,
+					     content_types[k], &length, NULL);
+
+      list = g_new (char *, 1 + length + 1);
+
+      i = 0;
+      if (remove)
+        list[i++] = g_strdup (desktop_id);
+      if (old_list)
+        {
+          for (j = 0; old_list[j] != NULL; j++)
+	    {
+	      if (g_strcmp0 (old_list[j], desktop_id) != 0)
+	        list[i++] = g_strdup (old_list[j]);
+	    }
+        }
+      list[i] = NULL;
+  
+      g_strfreev (old_list);
+
+      if (list[0] == NULL || desktop_id == NULL)
+        g_key_file_remove_key (key_file,
+			       REMOVED_ASSOCIATIONS_GROUP,
+			       content_types[k],
+			       NULL);
+      else
+        g_key_file_set_string_list (key_file,
+				    REMOVED_ASSOCIATIONS_GROUP,
+				    content_types[k],
+				    (const char * const *)list, i);
+
+      g_strfreev (list);
+    }
+  
+  g_strfreev (content_types);  
+
   data = g_key_file_to_data (key_file, &data_size, error);
   g_key_file_free (key_file);
   
@@ -1308,7 +1431,7 @@ run_update_command (char *command,
 			   G_SPAWN_DO_NOT_REAP_CHILD,
 			   NULL, NULL, /* No setup function */
 			   &pid,
-			   NULL)) 
+			   &error)) 
 	  g_child_watch_add (pid, update_program_done, NULL);
 	else
 	  {
@@ -1499,6 +1622,40 @@ g_desktop_app_info_ensure_saved (GDesktopAppInfo *info,
   return TRUE;
 }
 
+static gboolean
+g_desktop_app_info_can_delete (GAppInfo *appinfo)
+{
+  GDesktopAppInfo *info = G_DESKTOP_APP_INFO (appinfo);
+
+  if (info->filename)
+    return g_access (info->filename, W_OK) == 0;
+
+  return FALSE;
+}
+
+static gboolean
+g_desktop_app_info_delete (GAppInfo *appinfo)
+{
+  GDesktopAppInfo *info = G_DESKTOP_APP_INFO (appinfo);
+  
+  if (info->filename)
+    { 
+      if (g_remove (info->filename) == 0)
+        {
+          update_mimeapps_list (info->desktop_id, NULL, FALSE, FALSE, FALSE, NULL);
+
+          g_free (info->filename);
+          info->filename = NULL;
+          g_free (info->desktop_id);
+          info->desktop_id = NULL;
+
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 /**
  * g_app_info_create_from_commandline:
  * @commandline: the commandline to use
@@ -1572,6 +1729,8 @@ g_desktop_app_info_iface_init (GAppInfoIface *iface)
   iface->add_supports_type = g_desktop_app_info_add_supports_type;
   iface->can_remove_supports_type = g_desktop_app_info_can_remove_supports_type;
   iface->remove_supports_type = g_desktop_app_info_remove_supports_type;
+  iface->can_delete = g_desktop_app_info_can_delete;
+  iface->do_delete = g_desktop_app_info_delete;
 }
 
 static gboolean
@@ -1606,7 +1765,7 @@ g_app_info_get_all_for_type (const char *content_type)
 
   g_return_val_if_fail (content_type != NULL, NULL);
   
-  desktop_entries = get_all_desktop_entries_for_mime_type (content_type);
+  desktop_entries = get_all_desktop_entries_for_mime_type (content_type, NULL);
 
   infos = NULL;
   for (l = desktop_entries; l != NULL; l = l->next)
@@ -1629,6 +1788,22 @@ g_app_info_get_all_for_type (const char *content_type)
   return g_list_reverse (infos);
 }
 
+/**
+ * g_app_info_reset_type_associations:
+ * content_type: a content type 
+ *
+ * Removes all changes to the type associations done by
+ * g_app_info_set_as_default_for_type(), 
+ * g_app_info_set_as_default_for_extension(), 
+ * g_app_info_add_supports_type() of g_app_info_remove_supports_type().
+ *
+ * Since: 2.20
+ */
+void
+g_app_info_reset_type_associations (const char *content_type)
+{
+  update_mimeapps_list (NULL, content_type, FALSE, FALSE, FALSE, NULL);
+}
 
 /**
  * g_app_info_get_default_for_type:
@@ -1649,7 +1824,7 @@ g_app_info_get_default_for_type (const char *content_type,
 
   g_return_val_if_fail (content_type != NULL, NULL);
   
-  desktop_entries = get_all_desktop_entries_for_mime_type (content_type);
+  desktop_entries = get_all_desktop_entries_for_mime_type (content_type, NULL);
 
   info = NULL;
   for (l = desktop_entries; l != NULL; l = l->next)
@@ -2415,17 +2590,21 @@ append_desktop_entry (GList      *list,
 /**
  * get_all_desktop_entries_for_mime_type:
  * @mime_type: a mime type.
+ * @except: NULL or a strv list
  *
  * Returns all the desktop ids for @mime_type. The desktop files
  * are listed in an order so that default applications are listed before
  * non-default ones, and handlers for inherited mimetypes are listed
  * after the base ones.
  *
+ * Optionally doesn't list the desktop ids given in the @except 
+ *
  * Return value: a #GList containing the desktop ids which claim
  *    to handle @mime_type.
  */
 static GList *
-get_all_desktop_entries_for_mime_type (const char *base_mime_type)
+get_all_desktop_entries_for_mime_type (const char *base_mime_type,
+				       const char **except)
 {
   GList *desktop_entries, *removed_entries, *list, *dir_list, *tmp;
   MimeInfoCacheDir *dir;
@@ -2461,12 +2640,16 @@ get_all_desktop_entries_for_mime_type (const char *base_mime_type)
       g_strfreev (anc);
     }
   g_ptr_array_add (array, NULL);
-  mime_types = g_ptr_array_free (array, FALSE);
+  mime_types = (char **)g_ptr_array_free (array, FALSE);
 
   G_LOCK (mime_info_cache);
   
   removed_entries = NULL;
   desktop_entries = NULL;
+
+  for (i = 0; except != NULL && except[i] != NULL; i++)
+    removed_entries = g_list_prepend (removed_entries, g_strdup (except[i]));
+  
   for (i = 0; mime_types[i] != NULL; i++)
     {
       mime_type = mime_types[i];
@@ -2511,6 +2694,7 @@ get_all_desktop_entries_for_mime_type (const char *base_mime_type)
 
   g_strfreev (mime_types);
 
+  g_list_foreach (removed_entries, (GFunc)g_free, NULL);
   g_list_free (removed_entries);
   
   desktop_entries = g_list_reverse (desktop_entries);
