@@ -180,7 +180,7 @@ match_error (gint errcode)
 }
 
 static void
-translate_compile_error (gint *errcode, gchar **errmsg)
+translate_compile_error (gint *errcode, const gchar **errmsg)
 {
   /* Compile errors are created adding 100 to the error code returned
    * by PCRE.
@@ -252,6 +252,9 @@ translate_compile_error (gint *errcode, gchar **errmsg)
       break;
     case 129:
       *errcode = G_REGEX_ERROR_UNMATCHED_PARENTHESIS;
+      /* translators: '(?R' and '(?[+-]digits' are both meant as (groups of) 
+       * sequences here, '(?-54' would be an example for the second group.
+       */
       *errmsg = _("(?R or (?[+-]digits must be followed by )");
       break;
     case G_REGEX_ERROR_INEXISTENT_SUBPATTERN_REFERENCE:
@@ -362,7 +365,7 @@ translate_compile_error (gint *errcode, gchar **errmsg)
     case 45:
       /* These errors should not happen as we are using an UTF8-enabled PCRE
        * and we do not check if strings are valid */
-      g_warning (*errmsg);
+      g_warning ("%s", *errmsg);
       *errcode = G_REGEX_ERROR_COMPILE;
       break;
     default:
@@ -408,7 +411,11 @@ match_info_new (const GRegex *regex,
                      PCRE_INFO_CAPTURECOUNT, &capture_count);
       match_info->n_offsets = (capture_count + 1) * 3;
     }
+
   match_info->offsets = g_new0 (gint, match_info->n_offsets);
+  /* Set an invalid position for the previous match. */
+  match_info->offsets[0] = -1;
+  match_info->offsets[1] = -1;
 
   return match_info;
 }
@@ -492,6 +499,8 @@ g_match_info_next (GMatchInfo  *match_info,
 		   GError     **error)
 {
   gint opts;
+  gint prev_match_start;
+  gint prev_match_end;
 
   g_return_val_if_fail (match_info != NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -499,6 +508,9 @@ g_match_info_next (GMatchInfo  *match_info,
 
   opts = match_info->regex->match_opts | match_info->match_opts;
  
+  prev_match_start = match_info->offsets[0];
+  prev_match_end = match_info->offsets[1];
+
   match_info->matches = pcre_exec (match_info->regex->pcre_re,
 				   match_info->regex->extra,
 				   match_info->string,
@@ -537,6 +549,25 @@ g_match_info_next (GMatchInfo  *match_info,
       match_info->pos = match_info->offsets[1];
     }
 
+  /* it's possibile to get two identical matches when we are matching
+   * empty strings, for instance if the pattern is "(?=[A-Z0-9])" and
+   * the string is "RegExTest" we have:
+   *  - search at position 0: match from 0 to 0
+   *  - search at position 1: match from 3 to 3
+   *  - search at position 3: match from 3 to 3 (duplicate)
+   *  - search at position 4: match from 5 to 5
+   *  - search at position 5: match from 5 to 5 (duplicate)
+   *  - search at position 6: no match -> stop
+   * so we have to ignore the duplicates.
+   * see bug #515944: http://bugzilla.gnome.org/show_bug.cgi?id=515944 */
+  if (match_info->matches >= 0 &&
+      prev_match_start == match_info->offsets[0] &&
+      prev_match_end == match_info->offsets[1])
+    {
+      /* ignore this match and search the next one */
+      return g_match_info_next (match_info, error);
+    }
+
   return match_info->matches >= 0;
 }
 
@@ -1044,7 +1075,7 @@ g_regex_new (const gchar         *pattern,
 {
   GRegex *regex;
   pcre *re;
-  gchar *errmsg;
+  const gchar *errmsg;
   gint erroffset;
   gint errcode;
   gboolean optimize = FALSE;
@@ -1065,8 +1096,8 @@ g_regex_new (const gchar         *pattern,
       if (!support)
 	{
 	  msg = N_("PCRE library is compiled without UTF8 support");
-	  g_critical (msg);
-	  g_set_error (error, G_REGEX_ERROR, G_REGEX_ERROR_COMPILE, gettext (msg));
+	  g_critical ("%s", msg);
+	  g_set_error_literal (error, G_REGEX_ERROR, G_REGEX_ERROR_COMPILE, gettext (msg));
 	  return NULL;
 	}
 
@@ -1074,8 +1105,8 @@ g_regex_new (const gchar         *pattern,
       if (!support)
 	{
 	  msg = N_("PCRE library is compiled without UTF8 properties support");
-	  g_critical (msg);
-	  g_set_error (error, G_REGEX_ERROR, G_REGEX_ERROR_COMPILE, gettext (msg));
+	  g_critical ("%s", msg);
+	  g_set_error_literal (error, G_REGEX_ERROR, G_REGEX_ERROR_COMPILE, gettext (msg));
 	  return NULL;
 	}
 
@@ -1111,7 +1142,7 @@ g_regex_new (const gchar         *pattern,
 
   /* compile the pattern */
   re = pcre_compile2 (pattern, compile_options, &errcode,
-		      (const gchar **)&errmsg, &erroffset, NULL);
+		      &errmsg, &erroffset, NULL);
 
   /* if the compilation failed, set the error member and return 
    * immediately */
@@ -1158,7 +1189,7 @@ g_regex_new (const gchar         *pattern,
 
   if (optimize)
     {
-      regex->extra = pcre_study (regex->pcre_re, 0, (const gchar **)&errmsg);
+      regex->extra = pcre_study (regex->pcre_re, 0, &errmsg);
       if (errmsg != NULL)
         {
           GError *tmp_error = g_error_new (G_REGEX_ERROR,
